function DatatypeMapper() {
}

DatatypeMapper.prototype.mapHtml = function (property) {
    var parameter = property.Value;
    var key = property.Name;
    if (property.Options) {
        return DatatypeMapper_mapStrings(property.Name, property.Options, property.Value);
    }
    else if (parameter instanceof Array) {
        // parameter is a vector of strings
        if (parameter[0] instanceof String || typeof (parameter[0]) == 'string') {
            return DatatypeMapper_mapStrings(key, parameter);

        }
        // parameter is a vector of numbers
        else if (!isNaN(parameter[0])) {
            return DatatypeMapper_mapNumbers(key, parameter);
        }
        // parameter is a matrix of numbers
        else {
            return DatatypeMapper_mapMatrix(key, parameter);
        }

    }
    else if (typeof (parameter) == 'boolean') {
        return DatatypeMapper_mapBoolean(key, parameter);
    }
    // parameter is a number
    else if (!isNaN(parameter)) {
        return DatatypeMapper_mapNumber(key, parameter);
    }
    return 'No mapping for ' + typeof (parameter) + ' (Key: ' + key + ')';
}


function DatatypeMapper_mapStrings(key, param, selected) {
  // http://stackoverflow.com/questions/4814512/how-to-create-dropdown-list-dynamically-using-jquery
  var s = $('<select name="' + key + '"/>');
  for (var i = 0; i < param.length; i++) {
    $('<option />', { value: param[i], text: param[i] }).appendTo(s);
  }
  s.val(selected);
  return s;
}

function DatatypeMapper_mapNumber(key, param) {
  return $('<input type="text" name="' + key + '" />').val(param);
}

function DatatypeMapper_mapBoolean(key, param) {
  var s = $('<select name="' + key + '"/>');
  $('<option />', { value: 'true', text: 'true' }).appendTo(s);
  $('<option />', { value: 'false', text: 'false' }).appendTo(s);
  return s;
}

function DatatypeMapper_mapNumbers(key, param) {
  // http: //stackoverflow.com/questions/8749236/create-table-with-jquery-append
  var table = $('<table></table>').addClass('editableTable');
  var head = $('<thead></thead>');
  // create head elements
  var headerRow = $('<tr></tr>');
  $('<td />').text(key + ' Column').appendTo(headerRow);  
  headerRow.appendTo(head);
  head.appendTo(table);
  var body = $('<tbody></tbody>');
  for (var i = 0; i < param.length; i++) {
    var row = $('<tr></tr>');
    var td = $('<td />').appendTo(row);
    $('<input type="text" name="' + key + '_' + i + '" />').val(param[i]).appendTo(td);    
    row.appendTo(body);
  }
  body.appendTo(table);
  return table;
}

function DatatypeMapper_mapMatrix(key, param) {
  // http: //stackoverflow.com/questions/8749236/create-table-with-jquery-append
  var table = $('<table></table>').addClass('editableTable');
  var head = $('<thead></thead>');
  // create head elements
  var headerRow = $('<tr></tr>');
  for (var i = 0; i < param[0].length; i++) {
    $('<td />').text(key + ' Column ' + i).appendTo(headerRow);
  }
  headerRow.appendTo(head);
  head.appendTo(table);
  var body = $('<tbody></tbody>');
  for (var i = 0; i < param.length; i++) {
    var row = $('<tr></tr>');
    for (var j = 0; j < param[i].length; j++) {
      var td = $('<td />').appendTo(row);      
      $('<input type="text" name="' + key + '_' + i + '_' + j + '" />').val(param[i][j]).appendTo(td);      
    }
    row.appendTo(body);
  }
  body.appendTo(table);
  return table;

  /*var table = '<table class="editableTable"><thead><tr>';
  for (var i = 0; i < param[0].length; i++) {
    table += '<th>' + key + ' Column ' + (i + 1) + '</th>';
  }
  table += '</tr>';
  // foreach value entry (2d entry)
  for (var i = 0; i < param.length; i++) {
    table += '<tr>';
    for (var j = 0; j < param[i].length; j++) {
      table += '<td><input type="text" name="' + key + '_' + i + '_' + j + '" /></td>';
    }
    table += '</tr>';
  }
  return $*/
}

DatatypeMapper.prototype.mapString = function (value) {
  if (value == "true") return true;
  else if (value == "false") return false;
  else if (!isNaN(value)) return parseFloat(value.replace(',', '.'));
  return value;
}

DatatypeMapper.prototype.aggregateTables = function (data) {
    // aggregate tables:
    for (var dataKey in data) {
        var entry = data[dataKey];
        var tableEntries = {};
        var toRemove = [];
        for (var key in entry) {
            if (key.indexOf('_') != -1) {
                var splitted = key.split('_');
                var name = splitted[0];
                var row = parseInt(splitted[1]);
                if (!(name in tableEntries)) {
                    tableEntries[name] = []
                }

                if (splitted.length == 2) { // its a vector of elements
                    tableEntries[name].push(entry[key])
                } else if (splitted.length == 3) { // its a matrix of elements
                    column = parseInt(splitted[2]);
                    if (tableEntries[name].length < row + 1)
                        tableEntries[name].push([]);
                    tableEntries[name][row][column] = entry[key];
                }
                toRemove.push(key);
            }
        }

        for (var i = 0; i < toRemove.length; i++) {
            delete entry[toRemove[i]];
        }

        for (var key in tableEntries) {
            entry[key] = tableEntries[key];
        }
    }
}