﻿using System; 
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using HeuristicLab.Services.Optimization.Web.Models;
using HeuristicLab.Services.Optimization.Web.Helpers;
using HeuristicLab.Services.Optimization.ControllerService.Model;
using System.Web.Security;
using Mvc3TestApplication.Binders;
using HeuristicLab.Services.Optimization.ControllerService.Parsers;

namespace HeuristicLab.Services.Optimization.Web.Controllers
{
    [Authorize(Roles = "Web User")]
    public class ExperimentController : Controller
    {
        private ControllerServiceHelper ControllerService {
          get {
            var helper = new ControllerServiceHelper(Session["pw"] as string);
            if (!helper.Valid) {
              Response.Redirect("/Account/Logon?ReturnUrl=%2fExperiment");
            }
            return helper;
          }
        }

        //
        // GET: /Experiment/

        public ActionResult Index()
        {
          var model = new ExperimentViewModel();
          var scenarios = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            return service.GetOptimizationScenarioNames();
          });
          var experiments = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetExperiments(u);
          });
          foreach (var experiment in experiments)
            model.Experiments.Add(experiment);
          foreach (var scenario in scenarios)
            model.Scenarios.Add(scenario);

          Session["experiment"] = new Experiment();

          return View(model);
        }

        public ActionResult New() {
          return Index();
        }

        public Experiment Experiment { get { return Session["experiment"] as Experiment; } set { Session["experiment"] = value;} }

        [HttpPost]
        public JsonResult Save(ExperimentModel model) {
          var experiment = new Experiment() { Name = model.Name };
          var stack = new Stack<Node>();

          foreach (var node in model.Experiment.children)
            stack.Push(node);

          while (stack.Count > 0) {
            var currentNode = stack.Pop();
            // add children to stack
            if (currentNode.children != null)
              foreach (var node in currentNode.children)
                stack.Push(node);

            // TODO: Do we really need a tree of scenarios? Or is it a simple list?
            // push currentNode to experiment
            if (currentNode.title == "Experiment")
              continue;
            experiment.Algorithm.Add(new Algorithm() { Name = currentNode.title });
          }

          var ok = ControllerService.withControllerService<bool>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.SaveExperiment(u, experiment);
          });

          if (ok)
            return Json(new { success = true, data = model.Name });
          return Json(new { success = false, error = "Experiment already exists!" });
        }

        public ActionResult Edit() {
          var model = new ExperimentViewModel();
          var experiments = ControllerService.withControllerService<IEnumerable<string>>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetExperiments(u);
          });
          foreach (var experiment in experiments)
            model.Experiments.Add(experiment);
          return View(model);
        }

        public ActionResult DeleteExperiment(string experiment) {
          ControllerService.withControllerService<bool>((service) => {
            User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.DeleteExperiment(u, experiment);
          });
          return RedirectToAction("Edit");
        }

        [HttpPost]
        public JsonResult SaveExperiment([ExperimentJson] Experiment experiment) {
          return Json(
            new { 
              Success = ControllerService.withControllerService<bool>(service => {
                User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
                return service.SaveExperiment(u, experiment);
              })
            });
        }

        private JsonResult ComplexExperimentCase(string scenario) {
          // complex case, find children of experiment
          var model = new GetParametersModel() { Type = "Complex" };
          model.Subnames = new List<string>();

          var experiment = ControllerService.withControllerService<Experiment>(service => {
            User user = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
            return service.GetExperimentByName(user, scenario);
          });

          // TODO: Make a tree out of subnames!!
          if (experiment != null)
            foreach (var algo in experiment.Algorithm)
              model.Subnames.Add(algo.Name);

          return Json(model, JsonRequestBehavior.AllowGet);
        }

        [HttpGet]
        public JsonResult GetParameters(string scenario, string context, int index) {
          if (context == "Experiment" || context == null) {
            var optimizationScenario = ControllerService.withControllerService<OptimizationScenario>(service => {
              return service.GetOptimizationScenarioByName(scenario);
            });

            if (optimizationScenario != null) {
              var model = new GetParametersModel() { Type = "Basic" };
              model.Algorithm = optimizationScenario.FirstAlgorithm;
              model.Algorithm.Mapper = optimizationScenario.Id;
              var json = Json(model, JsonRequestBehavior.AllowGet);
              return json;
            }
            else {
              // complex case, find children of experiment
              var model = new GetParametersModel() { Type = "Complex" };
              model.Subnames = new List<string>();

              var experiment = ControllerService.withControllerService<Experiment>(service => {
                User user = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
                return service.GetExperimentByName(user, scenario);
              });

              // TODO: Make a tree out of subnames!!
              if (experiment != null)
                foreach (var algo in experiment.Algorithm)
                  model.Subnames.Add(algo.Name);

              return Json(model, JsonRequestBehavior.AllowGet);
            }
          }
          else {
            var experiment = ControllerService.withControllerService<Experiment>(service => {
              User u = new User() { Username = Membership.GetUser().UserName, Password = Session["pw"] as string };
              return service.GetExperimentByName(u, context);
            });

            // if child algorithm is a complex algorithm, return as such
            if (experiment.Algorithm[index].ChildAlgorithms.Count > 0) {
              var model = new GetParametersModel() { Type = "Complex" };
              model.Subnames = new List<string>();
              foreach (var algo in experiment.Algorithm[index].ChildAlgorithms)
                model.Subnames.Add(algo.Name);
              return Json(model, JsonRequestBehavior.AllowGet);
            }
            else {
              // otherwise we have the algorithms just below it
              var model = new GetParametersModel() { Type = "Basic" };
              model.Algorithm = experiment.Algorithm[index];
              model.Algorithm.Mapper = experiment.Algorithm[index].Name;
              var json = Json(model, JsonRequestBehavior.AllowGet);
              return json;
            }
          }
        }
    }
}
